@@ -19,28 +19,24 @@ import tracing
 import larch
 
 
-class BadKeySize(Exception):
+class MetadataMissingKey(larch.Error):
 
-    '''Exception for a bad key size.'''
+    def __init__(self, key_name):
+        self.msg = 'larch forest metadata missing "%s"' % key_name
+
+
+class BadKeySize(larch.Error):
 
     def __init__(self, store_key_size, wanted_key_size):
         self.msg = ('Node store has key size %s, program wanted %s' %
                     (store_key_size, wanted_key_size))
-    
-    def __str__(self):
-        return self.msg
 
 
-class BadNodeSize(Exception):
-
-    '''Exception for a bad node size.'''
+class BadNodeSize(larch.Error):
 
     def __init__(self, store_node_size, wanted_node_size):
         self.msg = ('Node store has node size %s, program wanted %s' %
                     (store_node_size, wanted_node_size))
-    
-    def __str__(self):
-        return self.msg
 
 
 class Forest(object):
@@ -137,7 +133,6 @@ class Forest(object):
         
         '''
         tracing.trace('committing forest')
-        self.node_store.push_upload_queue()
         self.node_store.set_metadata('last_id', self.last_id)
         root_ids = ','.join('%d' % t.root.id 
                             for t in self.trees 
@@ -146,12 +141,12 @@ class Forest(object):
         self.node_store.set_metadata('key_size', 
                                      self.node_store.codec.key_bytes)
         self.node_store.set_metadata('node_size', self.node_store.node_size)
-        self.node_store.save_metadata()
         self.node_store.save_refcounts()
+        self.node_store.commit()
 
 
-def open_forest(key_size=None, node_size=None, codec=None, node_store=None, 
-                **kwargs):
+def open_forest(allow_writes=None, key_size=None, node_size=None, codec=None, 
+                node_store=None, **kwargs):
     '''Create or open a forest.
     
     ``key_size`` and ``node_size`` are retrieved from the forest, unless
@@ -168,6 +163,8 @@ def open_forest(key_size=None, node_size=None, codec=None, node_store=None,
     '''
 
     tracing.trace('opening forest')
+    
+    assert allow_writes is not None
 
     codec = codec or larch.NodeCodec
     node_store = node_store or larch.NodeStoreDisk
@@ -177,10 +174,12 @@ def open_forest(key_size=None, node_size=None, codec=None, node_store=None,
         # For this, we can use any values for node and key sizes,
         # since we won't be accessing nodes or keys.
         c_temp = codec(42)
-        ns_temp = node_store(42, c_temp, **kwargs)
-        
-        assert 'key_size' in ns_temp.get_metadata_keys()
-        assert 'node_size' in ns_temp.get_metadata_keys()
+        ns_temp = node_store(False, 42, c_temp, **kwargs)
+
+        if 'key_size' not in ns_temp.get_metadata_keys():
+            raise MetadataMissingKey('key_size')
+        if 'node_size' not in ns_temp.get_metadata_keys():
+            raise MetadataMissingKey('node_size')
         
         if key_size is None:
             key_size = int(ns_temp.get_metadata('key_size'))
@@ -188,7 +187,7 @@ def open_forest(key_size=None, node_size=None, codec=None, node_store=None,
             node_size = int(ns_temp.get_metadata('node_size'))
     
     c = codec(key_size)
-    ns = node_store(node_size, c, **kwargs)
+    ns = node_store(allow_writes, node_size, c, **kwargs)
     
     def check_size(keyname, wanted, exception):
         if keyname not in ns.get_metadata_keys():
